#!/usr/bin/env python
# -*- coding: utf-8 -*-

import csv
import os
from typing import Dict, Iterator

import click
from mako.template import Template

from .sendmail import connect_smtp, create_email, html_to_text


def read_env() -> Dict[str, str]:
    def get_value(key: str) -> str:
        value = os.getenv(key)
        if value is None:
            raise Exception(f'{key} environment variable not set.')
        return value

    return {
        key: var_type(get_value(key.upper()))
        for key, var_type in (
            ('smtp_username', str),
            ('smtp_password', str),
            ('smtp_host', str),
            ('smtp_port', int),
        )
    }


def open_listing(filename: str) -> Iterator[Dict[str, str]]:
    with open(filename) as f:
        yield from csv.DictReader(f)


@click.command()
@click.argument('template_file', type=click.Path())
@click.argument('listing_file', type=click.Path())
@click.option('--dry-run', is_flag=True, help="Don't do anything.")
@click.option('--sender-name', default='', help='Sender name shown in emails.')
@click.option('--sender-email',
              default=None,
              help='Sender email (defaults to env SMTP_USERNAME)')
@click.option('--subject',
              'subject_template_str',
              required=True,
              help='Emails subject template.')
@click.option('--verbose', is_flag=True, help='Verbose mode.')
def main(template_file, listing_file, dry_run, sender_name, sender_email,
         subject_template_str, verbose):
    email_template = Template(filename=template_file)
    subject_template = Template(subject_template_str)
    env = read_env()

    if not dry_run:
        smtp_conn = connect_smtp(host=env['smtp_host'],
                                 port=env['smtp_port'],
                                 username=env['smtp_username'],
                                 password=env['smtp_password'])

    for i, row in enumerate(open_listing(listing_file)):
        subject = subject_template.render(**row)
        email_html = email_template.render(**row)

        if verbose:
            print('\x1b[32mSubject\x1b[0m:', subject)
            print(email_html)

        print(f"{i}\x1b[33m::\x1b[0m{row['email']}\x1b ", flush=True)

        email = create_email(
            sender_name=sender_name,
            sender_addr=sender_email or env['smtp_username'],
            recipient_addr=(row['email'], ),
            subject=subject,
            html=email_html,
            text=html_to_text(email_html),
        )

        if not dry_run:
            smtp_conn.send_message(email)
            print('sent.')
        else:
            print()


if __name__ == '__main__':
    main()



import email.utils
import re
import smtplib
from email.header import Header
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from typing import List, Union

ENCODING = 'utf-8'


def create_email(sender_name: str, sender_addr: str, recipient_addr: List[str],
                 subject: str, html: Union[str, None], text: Union[str, None]):
    msg = MIMEMultipart()

    sender = sender_addr if not sender_name else email.utils.formataddr(
        (Header(sender_name, ENCODING).encode(), sender_addr))

    msg['From'] = sender
    msg['To'] = email.utils.COMMASPACE.join(recipient_addr)
    msg['Subject'] = Header(subject, ENCODING)
    msg['Date'] = email.utils.formatdate()

    msg_related = MIMEMultipart('related')
    msg.attach(msg_related)

    msg_alternative = MIMEMultipart('alternative')
    msg_related.attach(msg_alternative)

    if text:
        msg_text = MIMEText(text.encode(ENCODING), 'plain', ENCODING)
        msg_alternative.attach(msg_text)

    if html:
        msg_html = MIMEText(html.encode(ENCODING), 'html', ENCODING)
        msg_alternative.attach(msg_html)

    return msg


def connect_smtp(host: str, port: int, username: str, password: str):
    conn = smtplib.SMTP_SSL(host, port)
    conn.login(username, password)

    return conn


def html_to_text(text: str) -> str:
    return re.sub('<[^<]+?>', '', str)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os.path
from pathlib import Path

from setuptools import setup


def readme_text() -> str:
    projet_root_path = os.path.abspath(os.path.dirname(__file__))
    return Path(projet_root_path, 'README.md').read_text()


setup(
    name='bulk-mail-sender',
    version='0.1.1',
    description='Send emails in bulk with a CSV listing and a email template.',
    author='Clément Martinez',
    author_email='clementmartinezdev@gmail.com',
    url='https://github.com/moverest/bulk-mail-sender',
    long_description=readme_text(),
    long_description_content_type='text/markdown',
    install_requires=('click', 'mako'),
    packages=('bulk_mail_sender', ),
    entry_points={
        'console_scripts': [
            'bulk-mail-sender=bulk_mail_sender.main:main',
        ]
    },
)
